@@ -3,6 +3,9 @@ import gnome.ui
 import gtk
 
 
+#
+# Class for the configuration widgets of a sensor.
+#
 class SensorConfigurator(gtk.VBox):
 
     CHANGE_ENTRY = 0
@@ -11,6 +14,7 @@ class SensorConfigurator(gtk.VBox):
     CHANGE_CHECKBOX = 3
     CHANGE_FONT = 4
     CHANGE_COLOR = 5
+    CHANGE_FILE = 6
 
 
     def __init__(self, set_config, get_config):
@@ -38,6 +42,9 @@ class SensorConfigurator(gtk.VBox):
 
 
 
+    #
+    # Adds a line of widgets to the configurator. The line can be indented.
+    #
     def __add_line(self, indent, w1, w2 = None):
 
         self.__lines += 1
@@ -53,7 +60,7 @@ class SensorConfigurator(gtk.VBox):
                                 gtk.EXPAND | gtk.FILL, 0, 0, y)
 
         else:
-            self.__table.attach(w1, 0, 1, self.__lines - 1, self.__lines,
+            self.__table.attach(w1, 0, 2, self.__lines - 1, self.__lines,
                                 gtk.EXPAND | gtk.FILL, 0, x, y)
 
 
@@ -90,10 +97,18 @@ class SensorConfigurator(gtk.VBox):
             alpha = ("0" + hex(a / 256)[2:-1])[-2:]
             value = "#" + red + green + blue #+ alpha
 
+        elif (mode == self.CHANGE_FILE):
+            file = src.get_full_path(file_must_exist = gtk.TRUE)
+            print "FILE:", file
+
         self.__set_config(property, value)
 
 
 
+    #
+    # Sets/returns the name of this configurator. That name will appear in the
+    # notebook tab.
+    #
     def set_name(self, name): self.__name = name
     def get_name(self): return self.__name
 
@@ -127,7 +142,7 @@ class SensorConfigurator(gtk.VBox):
 
 
 
-    def add_entry(self, label, property, help):
+    def add_entry(self, label, property, help, passwd = 0):
 
         lbl = gtk.Label(label)
         lbl.show()
@@ -137,6 +152,10 @@ class SensorConfigurator(gtk.VBox):
         align.show()
         align.add(lbl)
 
+        if (passwd):
+            entry.set_visibility(gtk.FALSE)
+            entry.set_invisible_char(unichr(0x2022))
+
         self.__tooltips.set_tip(entry, help)
         self.__add_line(1, align, entry)
 
@@ -217,8 +236,6 @@ class SensorConfigurator(gtk.VBox):
 
         btn = gnome.ui.FontPicker()
         btn.set_title("")
-        btn.set_preview_text("AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRr"
-                             "SsTtUuVvWwXxYyZz")
         btn.fi_set_use_font_in_label(gtk.TRUE, 12)
         btn.set_mode(gnome.ui.FONT_PICKER_MODE_FONT_INFO)
         btn.show()
@@ -252,3 +269,45 @@ class SensorConfigurator(gtk.VBox):
 
         self.__tooltips.set_tip(btn, help)
         self.__add_line(1, align, btn)
+
+
+
+    def add_file_selector(self, label, property, help):
+
+        def open_filedialog(self, on_ok, entry):
+            def fd_hide(src, fselector): fselector.destroy()
+            d = gtk.FileSelection()
+            d.show()
+            d.ok_button.connect("clicked", on_ok, d, entry)
+            d.cancel_button.connect("clicked", fd_hide, d)
+
+        def on_ok(src, fselector, entry):
+            fname = fselector.get_filename()
+            entry.set_text(fname)
+            fselector.destroy()
+
+        lbl = gtk.Label(label)
+        lbl.show()
+        align = gtk.Alignment()
+        align.show()
+        align.add(lbl)
+
+        hbox = gtk.HBox()
+        hbox.show()
+        entry = gtk.Entry()
+        entry.show()
+        hbox.pack_start(entry, gtk.TRUE, gtk.TRUE, 0)
+
+        btn = gtk.Button(stock = gtk.STOCK_OPEN)
+        btn.show()
+        hbox.pack_end(btn, gtk.TRUE, gtk.TRUE, 4)
+
+        btn.connect("clicked", open_filedialog, on_ok, entry)
+
+        value = self.__get_config(property)
+        entry.set_text(value)
+        entry.connect("changed", self.__on_change, property,
+                      self.CHANGE_ENTRY)
+
+        self.__tooltips.set_tip(hbox, help)
+        self.__add_line(1, align, hbox)
